<!DOCTYPE html>
<html lang="en">
<head>
	<meta charset="UTF-8">
	<meta content="width=device-width, height=device-height, initial-scale=1, minimal-ui" name="viewport" />
	<title>Share File WEB UI v{{ data.version }}</title>
	<link rel="shortcut icon" href="{{ url_for('static', filename='img/share.png')}}" />
	<link href="{{ url_for('static', filename='css/dropzone.css')}}?v={{data.version}}" rel="stylesheet" type="text/css" />
	<link href="{{ url_for('static', filename='css/app.css')}}?v={{data.version}}" rel="stylesheet" type="text/css" />
	<script type="application/javascript" src="{{ url_for('static', filename='js/dropzone.js')}}?v={{data.version}}"></script>
</head>
<body>
	<div class="app">
		<h1>Share File WEB UI <span class="user">({{ data.user }})</span> <span class="version">v{{data.version}}</span></h1>
		<div id="main-container"></div>
	</div>
	<script type="application/javascript" src="{{ url_for('static', filename='js/sharefile-webui.js')}}?v={{data.version}}"></script>
</body>
</html>
